/*

Name: XCheck

Version: 1.0
 
Description: an implementation of XCheck-Java for command-line query processors.

Author: Copyright (C) 2007 Massimo Franceschet (massimo.franceschet@dimi.uniud.it) - Dipartimento di Matematica e Informatica, Universita' degli Studi di Udine, Via delle Scienze, 206 - 33100 Udine (Italy)

License: GNU General Public License (GPL)    
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

import java.io.*;
import org.w3c.dom.*;
import org.w3c.dom.ls.*;
import javax.xml.*;
import javax.xml.parsers.*;
import javax.xml.xpath.*;
import org.xml.sax.SAXException;


/******************************************************
Implement XCheck-Java for command-line query processors
*******************************************************/
public class XCheck extends XCheckVoid {

  /***********************
  Parse the input document 
  ************************/
  public Document parseInputDocument(int j, String target) {
      
    return null;
    
  }

        
  /**************
  Execute a query 
  ***************/
  public Object executeInputQuery(int j, Document document, String query)  {
    

    // set options
    parseOption = "no";
    executeOption = "yes";
    serializeOption = "no";
    overallOption = "no";
    
    // Prepare the command 
    String fullQueryFileName = null;
    File temp = null;
    if (! queryFileName[j].equals("")) {
        // the query is specified in a separated file
      fullQueryFileName = xcheckHome + experimentFolder + 
          "queries/" + queryFileName[j]; 
    } else {
        // the query is given in the experiment document. 
        // Use a temporary file to store the query
      try {
        File dir = new File(xcheckHome + experimentFolder);
        temp = File.createTempFile(queryID[j] + "_temp", ".xql", dir);
        temp.deleteOnExit();
      } catch (IOException ioe) {ioe.printStackTrace();}
      PrintWriter tempPW = null;
      try {
        tempPW = new PrintWriter(temp);
      } catch (IOException ioe) {ioe.printStackTrace();}
      tempPW.print(querySyntax[j]);
      tempPW.close();
      fullQueryFileName = xcheckHome + experimentFolder + temp.getName();
    }
      
    String fullQueryTargetFileName = xcheckHome + experimentFolder + 
        "documents/" + queryTargetFile[j]; 
    String resultFileName = 
        xcheckHome + experimentFolder + "outcome_" + engineID + 
        SEP + "answers" + SEP + "answer" + "_" + queryID[j] + ".xml";

    String command1 = engineExecutable.replace("#query", fullQueryFileName);
    String command2 = command1.replace("#document", fullQueryTargetFileName);
    String command = command2.replace("#result", resultFileName);
    
    
    
    Process proc = null;
    try {
      proc = Runtime.getRuntime().exec(command);
    } catch (IOException ioe) {
      printErrorMessage("There was a problem while executing query " + 
          queryID[j], ioe.toString());
      errorQuery[j] = true;
      errorMessageQuery[j] = ioe.toString();
    }
    if (proc != null) { // the process exists
      int exit = 0;
      try {
        exit = proc.waitFor();
        if (exit != 0) {
          errorQuery[j] = true;
          errorMessageQuery[j] = getErrorMessage(proc.getErrorStream());
          printErrorMessage("There was a problem while executing query " +  
              queryID[j], errorMessageQuery[j]);
        }
      } 
      catch (InterruptedException ie) {
        printErrorMessage("There was a problem while executing query " +  
            queryID[j], ie.toString());
        errorQuery[j] = true;
        errorMessageQuery[j] = ie.toString();
      }
    }
    return null;
  }
  
  
  /*************************
  Serialize the query result 
  **************************/
  public void serializeResult(int j, Object result, String resultFileName,
                              Document document) {}
  
  
  
  /************************************************
  Run XCheck-Java for command-line query processors
  *************************************************/
  public static void main(String[] args) {
    
    
    
    // create an instance of XCheck
    XCheck xcheck = new XCheck();
    
    // run XCheck's evaluation
    xcheck.check(args);
    
  }


}

