/*

Name: XCheckBaseX

Version: 1.0
 
Description: an implementation of XCheck-Java with BaseX. 

Author: Copyright (C) 2007 Massimo Franceschet (massimo.franceschet@dimi.uniud.it) and Christian Gruen (christian.gruen@uni-konstanz.de)

License: GNU General Public License (GPL)    
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

import java.io.*;
import org.w3c.dom.*;
import org.w3c.dom.ls.*;
import javax.xml.*;
import javax.xml.parsers.*;
import javax.xml.xpath.*;
import org.xml.sax.SAXException;
import de.unikn.inf.dbis.basex.*;
import de.unikn.inf.dbis.basex.io.*;


/*******************************
Implement XCheck-Java with BaseX
********************************/
public class XCheckBaseX extends XCheckVoid {

  // BaseX core instance
  public BaseXCore core;

  /*********************************************************************
  Run XCheck-Java with BaseX by calling the method check inherited from 
  class XCheckVoid
  *********************************************************************/
  public static void main(String[] args) {
    
    // creating XCheckBaseX instance
    XCheckBaseX basex = new XCheckBaseX();
    // creating BaseX core instance
    basex.core = new BaseXCore();
    // ...whitespaces between tags are chopped if this property is set to true
    Prop.chopped = false;
    // ...all entities are parsed (as BaseX doesn't handle DTDs yet, it
    // should be set to false if unknown entities exist in the document.)
    Prop.entity = true;
    // ...output will be well-formed xml
    Prop.wellformed = true;
    // ...use disk-based processing (main-memory will be a little bit faster) 
    Prop.mmemory = false;
    // running the experiment
    basex.check(args);
    
  }

  /***********************
  Parse the input document 
  ************************/
  public Document parseInputDocument(int j, String target) {
      
    // Builds/loads the database instance.
    // Three commands can be used here:
    // - CHECK loads an existing database instance; if none is found,
    //   a new instance is created.
    // - CREATE always creates a new database instance
    // - LOAD loads an existing database instance or returns false
    if(! core.process(Command.CHECK, target)) {
      printErrorMessage("There was a problem while parsing document: " + 
          target, core.getInfo()); 
      errorDocument[j] = true;
      errorMessageDocument[j] = core.getInfo();
    }
    return null;
  }

        
  /**************
  Execute a query 
  ***************/
  public Object executeInputQuery(int j, Document document, String query)  {
    
    // Execute query without serializing the result
    if(! core.process(Command.SHOW, query)) {
      printErrorMessage("There was a problem while parsing query: " + 
          query, core.getInfo()); 
      errorMessageQuery[j] = core.getInfo();
      errorQuery[j] = true;
    }
    return null;
  }
  
  
  /*************************
  Serialize the query result 
  **************************/
  public void serializeResult(int j, Object result, String resultFileName,  
                              Document document) {
    
    // PrintOutput is a BaseX specific output stream
    PrintOutput out = null;                            
    try {
        out = new PrintOutput(resultFileName);
    } catch (IOException ioe) {ioe.printStackTrace();}
    core.getResult(out);
    try {
      out.close();
    } catch (IOException ioe) {ioe.printStackTrace();}
  }


}

